import { DataPoint, DataPointMeta } from "../types/types";
import { Query } from "../index";
import { AddPointsOptions, Cell, CellValue, Key, KeyMode, Model, ModelizerOptions, ProcessColumnCb, ProcessRowCb, RenderTableCb } from "./modelizer-types";
/**
 * Modelizer class needs some datapoints to work. Each datapoint will add
 * a key to #keys.row and #keys.col. All #keys will define the rows and
 * columns of the output table. Rows and columns can be added, removed, sorted
 * and modelized.
 */
export default class Modelizer {
    #private;
    /**
     * If strict mode is 'true', no rows/columns/keys will be added automatically.
     * In lax mode, any non-existing string passing #parseCellKey will create
     * a new entry, which makes it easy to add new rows or columns.
     */
    strict: boolean;
    /**
     * The parent class for this Modelizer instance
     */
    query: Query;
    constructor(options?: ModelizerOptions, query?: Query);
    /**
     * Add an array of Datapoints to Modelizer. Each datapoint will create a
     * corresponding row- or column key, if not existing already.
     * Each point will create a cell.
     * If a point is added to an existing cell (because row and col key equal an
     * existing keypair), the point will push existing points in the target cell.
     * @param points An array of datapoints
     * @param options Specify where the target row-/column-keys are coming from
     * @return {this}
     */
    addPoints(points: DataPoint[], options?: AddPointsOptions): this;
    /**
     * Add a single Datapoint to the table. If the point matches an already
     * existing Cell by row and column key, the point will be added to the
     * stack of points of the matching Cell. Otherwise, a new row- and/or
     * column-key will be created according to the given datapoint.
     * @param point A datapoint
     * @param options Specify where the target row-/column-keys are coming from
     * @returns {this}
     */
    addPoint(point: DataPoint, options?: AddPointsOptions): this;
    /**
     * Apply a callback to each cell of the current table.
     * @returns {this}
     * @param cb The callback to run on each cell
     */
    process(cb: RenderTableCb): this;
    /**
     * Pass a callback to run on each row or column by given mode.
     * @param mode KeyMode is either 'row' or 'col'
     * @param cb The callback to run on each row
     */
    processByMode(mode: KeyMode, cb: ProcessRowCb | ProcessColumnCb): this;
    /**
     * Pass a callback to run on each row.
     * @param cb The callback to run on each row
     */
    processRows(cb: ProcessRowCb): this;
    /**
     * Pass a callback to run on each column.
     * @param cb The callback to run on each column
     */
    processColumns(cb: ProcessColumnCb): this;
    /**
     * Retreive the current keys for row or column.
     * @param mode Pass "row" or "col" to specify the target dimension.
     * @returns {string[]} Array of strings containing the keys.
     */
    getKeys(mode: KeyMode): string[];
    /**
     * Pass a string key to append a new row with that name.
     * @param key
     * @returns {number} Column index of the created row.
     */
    addRow(key: string): number;
    /**
     * Pass a string key to append a new column with that name.
     * @param key
     * @returns {number} Column index of the created column.
     */
    addColumn(key: string): number;
    setCellValueByMode(mode: KeyMode, k1: Key, k2: Key, value: CellValue): Cell;
    /**
     * Pass a {CellValue} to a target {Cell}.
     * @param r Pass a number or a string to determine the target cell row.
     * @param c Pass a row or column key to target the cell.
     * @param value A string or a number to alter the cell value.
     * @return {Cell}
     */
    setCellValue(r: Key, c: Key, value: CellValue): Cell;
    /**
     * Pass a Datapoint to a target Cell. The point will not change the
     * target cell value.
     * @param r Pass a number or a string to determine the target cell row.
     * @param c Pass a row or column key to target the cell.
     * @param point The point to push the cell's point stack with.
     */
    pushCellPoints(r: Key, c: Key, point: DataPoint): Cell;
    /**
     * Retreive a rows- or columns-model object for the given selector.
     * @param mode Pass 'row' or 'column'.
     * @param key Pass a number or a string to select the target row/column.
     * @return A model row/column object containing all cells.
     */
    getByMode(mode: KeyMode, key: Key): Model;
    getRow(key: Key): Model;
    getColumn(key: Key): Model;
    findOrCreateModel(models: Model[], mode: KeyMode, key: Key): Model;
    /**
     * Retreive a row or column model object for the given selector.
     * @param key Pass a number or a string to set the target key.
     * @param mode Specify "row" or "column"
     * @return A model object containing all cells.
     */
    createModel(key: string, mode: KeyMode): Model;
    getCellByMode(mode: KeyMode, i1: Key, i2: Key): Cell;
    /**
     * Retrieve a cell by row and column key.
     * If no cell matches the given keys, a new cell will be created (non-strict).
     * In strict mode, Modelizer will throw an error.
     * @param r Set a numeric or string value to address the target row
     * @param c Set a numeric or string value to address the target column
     * @returns {Cell}
     */
    getCell(r: Key, c: Key): Cell;
    setCellByMode(mode: KeyMode, k1: Key, k2: Key, cell?: Cell): Cell;
    /**
     * Update a Cell object at the given keys or create one.
     * In strict mode, Modelizer will throw an error when there are no matching keys.
     * @param rowKey Set a numeric or string value to address the target row
     * @param colKey Set a numeric or string value to address the target column
     * @param cell Optionally set a numeric or string value to address the target column
     * @returns {Cell}
     */
    setCell(rowKey: Key, colKey: Key, cell?: Cell): Cell;
    getCells(): Cell[];
    setMeta(key: string, value: DataPointMeta["value"]): void;
    getMeta(key: string): DataPointMeta;
    getNestedMeta(key: string, label: string): import("../index").ResultCell;
    getMetas(): DataPointMeta[];
    /**
     * Get the first of all DataPoints. This is helpful if you need general tags
     * or meta information and if you can assure that all active datapoints
     * hold the same information as Point zero.
     * @returns {DataPoint}
     */
    getFirstPoint(): DataPoint | null;
    /**
     * Specify "row" or "column" and an array of keys to filter and sort the target
     * dimension of current table.
     * @param mode
     * @param keys
     */
    sort(mode: KeyMode, keys: Key[]): this;
    /**
     * Transpose the result by switching rows and columns.
     */
    transpose(): void;
    /**
     * Write the contents of current table to the console.
     * @param firstColSize Specify the number of characters for label width.
     * @param colSize Number of characters for body column width.
     * @param rows Restrict the outputted rows to a given set of keys.
     * @param cols Restrict the outputted columns to a given set of keys.
     * @param renderCell Pass a callback to display customized information.
     */
    dump(firstColSize?: number, colSize?: number, rows?: Key[], cols?: Key[], renderCell?: (cell: Cell) => any): void;
}
