"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("./client");
const helper_1 = require("./helper");
const lodash_1 = __importDefault(require("lodash"));
const points_1 = __importDefault(require("./points"));
const modelizer_1 = __importDefault(require("./modelizer/modelizer"));
const convert_1 = __importDefault(require("./convert"));
const keys_1 = __importDefault(require("./keys"));
class Query {
    constructor(prisma) {
        this.selector = [[]];
        this.dataTagSelector = [[]];
        this.nonGreedySelector = [];
        this.maxSheets = 150;
        this.prisma = prisma;
        return this;
    }
    static run(options) {
        const prisma = options.prisma || new client_1.PrismaClient();
        const query = new Query(prisma).setOptions(options);
        return query.compute();
    }
    setOptions(options) {
        this.maxSheets = options.maxSheets ?? this.maxSheets;
        this.grid = options.grid || {
            modify: [],
            transform: [],
            options: {},
        };
        this.nonGreedySelector =
            options.nonGreedySelector || this.nonGreedySelector;
        this.cache = options.cache;
        this.dataTagSelector = options.dataTagSelector;
        this.selector = options.selector || this.selector;
        return this;
    }
    async compute() {
        if (this.dataTagSelector) {
            this.selector = await this.convertDataTagSelector(this.dataTagSelector);
        }
        const selector = this.selector;
        const tagIds = await this.parseSelector(selector).catch(() => {
            throw "Parse Selector failed";
        });
        const { points, sheets, tags, inputKeys } = await this.getRawResult(tagIds);
        const modelizer = new modelizer_1.default({
            points,
            strict: false,
        }, this);
        const errors = [];
        const dumpedData = [];
        const dump = (data, section) => {
            dumpedData.push({
                section,
                data,
            });
        };
        if (this.grid.transform) {
            await this.transformResult(this.grid.transform, modelizer, inputKeys, errors, dump);
        }
        return {
            modelizer,
            sheets,
            tags,
            inputKeys: inputKeys.getInputKeys(),
            convert: (tmpModelizer) => new convert_1.default(tmpModelizer || modelizer),
            isValid: () => !!modelizer.getFirstPoint(),
            visibleKeys: {
                row: modelizer.getKeys("row"),
                column: modelizer.getKeys("column"),
            },
            errors,
            dumpedData,
        };
    }
    async getRawResult(allTagIds) {
        const modifiedDataPoints = [];
        const usedTags = [];
        const usedDatasheets = [];
        const usedDatapoints = [];
        const inputKeys = new keys_1.default();
        for (const level in allTagIds) {
            const tagIds = allTagIds[level];
            const isNonGreedy = this.nonGreedySelector.includes(Number(level));
            let dataSheets = [];
            let selectionTags = [];
            if (this.cache?.exists(tagIds, isNonGreedy)) {
                const cachedObject = this.fromCache(this.cache.get(tagIds, isNonGreedy));
                selectionTags = cachedObject.tags;
                dataSheets = cachedObject.sheets;
            }
            else {
                selectionTags = await this.getTagInfo(tagIds);
                dataSheets = await this.findSheets(selectionTags, isNonGreedy);
                this.cache?.set(tagIds, isNonGreedy, lodash_1.default.cloneDeep({
                    sheets: dataSheets,
                    tags: selectionTags,
                }));
            }
            const datapoints = this.extractDataPoints(dataSheets);
            inputKeys.addPoints(datapoints);
            usedDatapoints.push(...datapoints);
            usedTags.push(selectionTags);
            usedDatasheets.push(...dataSheets);
            const pointsCls = this.modifyDataPoints(datapoints, Number(level));
            modifiedDataPoints.push(...pointsCls.points);
        }
        return {
            inputKeys,
            points: modifiedDataPoints,
            tags: usedTags,
            sheets: usedDatasheets,
        };
    }
    fromCache(cached) {
        return lodash_1.default.cloneDeep(cached);
    }
    async getTagInfo(tagIds) {
        return this.prisma.tag.findMany({
            where: {
                id: {
                    in: tagIds,
                },
            },
        });
    }
    async findSheets(tags, isNonGreedy) {
        let clause = (0, helper_1.getNestedClause)(tags);
        if (!clause)
            return [];
        let sheets = await this.prisma.sheet.findMany({
            where: clause,
            include: {
                tags: true,
            },
            skip: 0,
            take: this.maxSheets,
        });
        if (sheets.length === this.maxSheets) {
            // throw "Too many sheets. Add more tags to selection.";
            return [];
        }
        if (sheets.length && isNonGreedy) {
            sheets = this.filterSheets(sheets);
        }
        return this.parseSheets(sheets);
    }
    parseSheets(sheets) {
        const dataSheets = sheets.map((sheet) => {
            return {
                id: sheet.id,
                tags: sheet.tags.map((tag) => {
                    return {
                        id: tag.id,
                        value: tag.name,
                        categoryId: tag.categoryId,
                    };
                }),
                rows: JSON.parse(sheet.rows),
                columns: JSON.parse(sheet.columns),
                data: JSON.parse(sheet.data),
                meta: JSON.parse(sheet.meta),
            };
        });
        return dataSheets;
    }
    /*
     * Filters sheets with the least tag count by categoryId.
     * This will prevent mixing sheets with different set of tags.
     */
    filterSheets(sheets) {
        const nonGreedyIds = this.getSheetCategoryCount(sheets).map((count) => count.sheetId);
        return sheets.filter((sheet) => nonGreedyIds.includes(sheet.id));
    }
    getSheetCategoryCount(sheets) {
        const categoryCount = [];
        sheets.forEach((sheet) => {
            categoryCount.push({
                sheetId: sheet.id,
                categoryIds: sheet.tags.map((tag) => tag.categoryId),
            });
        });
        categoryCount.sort((a, b) => a.categoryIds.length - b.categoryIds.length);
        const smallestCount = categoryCount[0].categoryIds.length;
        return categoryCount.filter((categoryCount) => categoryCount.categoryIds.length <= smallestCount);
    }
    extractDataPoints(sheets) {
        const dataPoints = [];
        sheets.forEach((sheet) => {
            sheet.data.forEach((points, r) => {
                points.forEach((value, c) => {
                    dataPoints.push(points_1.default.dataPointFactory(sheet.rows[r], sheet.columns[c], sheet.tags, points_1.default.getDataPointMeta(sheet, r, c), value));
                });
            });
        });
        return dataPoints;
    }
    modifyDataPoints(dataPoints, level) {
        const modifiers = this.getDatapointModifiersByLevel(level);
        const points = new points_1.default(dataPoints);
        modifiers.forEach((modifier) => {
            if (modifier.cb) {
                modifier.cb(points);
            }
            if (modifier.callbacks) {
                modifier.callbacks.forEach((callback) => {
                    if (typeof callback === "function") {
                        callback(points);
                    }
                });
            }
        });
        return points;
    }
    getDatapointModifiersByLevel(level) {
        const modifiers = [];
        this.grid?.modify?.forEach((modifier) => {
            if (modifier.applyToLevel && modifier.applyToLevel.indexOf(level) > -1) {
                modifiers.push(modifier);
            }
            else if (!modifier.applyToLevel || modifier.applyToLevel.length === 0) {
                modifiers.push(modifier);
            }
        });
        return modifiers;
    }
    async parseSelector(selector) {
        if (selector[0][0] === undefined) {
            throw "Selection is empty";
        }
        if (typeof selector[0][0] === "number") {
            return selector;
        }
        throw "Invalid selector.";
    }
    async convertDataTagSelector(selector) {
        const tagIdSelector = [];
        for (const dataTag of selector) {
            const tagIds = await this.getTagIds(dataTag);
            tagIdSelector.push(tagIds);
        }
        return tagIdSelector;
    }
    async getTagIds(tags) {
        tags = await this.getTags(tags);
        return tags.map((tag) => tag.id);
    }
    async getTags(tags) {
        for (let i in tags) {
            let tag = tags[i];
            await this.setCategoryId(tag);
            await this.setTagId(tag);
        }
        return tags;
    }
    async setCategoryId(tag) {
        if (tag.categoryId)
            return;
        const categoryItem = await this.prisma.category.findFirst({
            where: {
                name: tag.category,
            },
        });
        if (!categoryItem) {
            throw new Error(`Category not found: ${tag.category}`);
        }
        tag.categoryId = categoryItem.id;
    }
    async setTagId(tag) {
        if (tag.id)
            return;
        const tagItem = await this.prisma.tag.findFirst({
            where: {
                categoryId: tag.categoryId,
                name: tag.value,
            },
        });
        if (!tagItem) {
            throw new Error(`Tag not found: ${tag.value} @ category: ${tag.category}`);
        }
        tag.id = tagItem.id;
    }
    async transformResult(transformations, modelizer, inputKeys, errors, dump) {
        for (const transform of transformations) {
            if (transform.modelize && typeof transform.modelize === "function") {
                const apply = await this.checkCondition(transform, modelizer);
                if (apply) {
                    const args = {
                        query: this,
                        inputKeys,
                        params: transform.params,
                        dump,
                    };
                    try {
                        await transform.modelize(modelizer, args);
                    }
                    catch (e) {
                        errors.push({
                            errorMessage: e.message,
                            lineNumber: e.lineNumber,
                            name: transform.name,
                            params: transform.params,
                            executionOrder: transform.executionOrder,
                            errorCode: transform.modelize.toString(),
                        });
                    }
                }
            }
        }
    }
    async checkCondition(transform, modelizer) {
        if (transform.condition && typeof transform.condition === "function") {
            if ((await transform.condition(modelizer)) !== true) {
                return false;
            }
        }
        if (!modelizer.getFirstPoint()) {
            return false;
        }
        return true;
    }
}
exports.default = Query;
//# sourceMappingURL=query.js.map