"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const query_1 = __importDefault(require("./query"));
const crypto_1 = __importDefault(require("crypto"));
const tagsCache_1 = __importDefault(require("./helper/tagsCache"));
class Store {
    constructor(prisma, options) {
        this.prisma = prisma;
        this.summary = {
            ids: [],
            deleted: [],
        };
        this.query = new query_1.default(this.prisma);
        this.options = {
            replaceExisting: true,
            filename: "unknown",
            userId: 1,
            statusTracker: (status) => {
                console.log(status.share);
            },
            tagsCache: new tagsCache_1.default(),
        };
        if (options) {
            this.options = Object.assign(this.options, options);
        }
        this.importId = 0;
        this.tagValueMaxLength = 256;
        this.categories = [];
        this.sheetKeys = {};
        this.tags = [];
        this.status = {
            current: 0,
            max: 0,
            share: 0,
            info: undefined,
            increment: () => {
                this.status.current++;
                this.status.share = Math.round((this.status.current / this.status.max) * 100);
                this.status.next(this.status);
            },
            next: this.options.statusTracker,
        };
    }
    async run(datasheets) {
        await this.runBefore();
        this.status.max = datasheets.length;
        this.status.info = "analyze datasheets";
        await this.getImport();
        await this.getCategories();
        await this.getTags();
        for (let i in datasheets) {
            this.status.info = "store datasheets";
            this.status.increment();
            await this.storeData(datasheets[i]);
        }
        // await this.prisma.$disconnect()
        return this.summary;
    }
    async runBefore() {
        if (this.options?.dropAllSheets === true) {
            const deleted = await this.prisma.sheet.deleteMany();
            console.log("Deleted " + String(deleted.count) + " sheets");
        }
        if (this.options?.dropAllTags === true) {
            const deletedTags = await this.prisma.tag.deleteMany();
            await this.options.tagsCache.init(this.prisma);
            console.log("Deleted " + String(deletedTags.count) + " tags");
        }
    }
    async getImport() {
        const newImport = await this.prisma.import.create({
            data: {
                file: this.options.filename,
                user: {
                    connect: {
                        id: this.options.userId,
                    },
                },
            },
        });
        this.importId = newImport.id;
    }
    async getCategories() {
        this.categories = await this.prisma.category.findMany();
    }
    async createCategory(name) {
        const newCat = await this.prisma.category.create({
            data: {
                name: name,
                color: "grey",
                icon: "settings",
            },
        });
        this.categories.push(newCat);
        return newCat;
    }
    async getTags() {
        await this.options.tagsCache.init(this.prisma);
        // this.tags = await this.prisma.tag.findMany();
    }
    async createTag(name, catId) {
        const exists = this.options.tagsCache.get(name, catId);
        if (exists) {
            return exists;
        }
        return this.options.tagsCache.create(name, catId);
        // const newTag = await this.prisma.tag.create({
        //   data: {
        //     name: name,
        //     category: {
        //       connect: { id: catId },
        //     },
        //   },
        // });
        // this.tags.push(newTag);
        // return newTag;
    }
    async storeData(datasheet) {
        await this.addCategoryIdToTags(datasheet.tags);
        await this.addTagIdsToTags(datasheet.tags);
        const sheetData = this.createSheetData(datasheet);
        await this.dedupTagKeys(sheetData);
        if (this.options?.replaceExisting) {
            await this.deleteExistingDatasheets(sheetData);
        }
        await this.createSheet(datasheet, sheetData);
    }
    createSheetData(datasheet) {
        const tagIds = datasheet.tags.map((tag) => {
            return { id: tag.id };
        });
        const tagInfo = datasheet.tags.map((tag) => tag.id);
        const { rowKey, columnKey, tagKey, sheetKey } = this.getSheetKeys(datasheet);
        return {
            columns: JSON.stringify(datasheet.columns),
            rows: JSON.stringify(datasheet.rows),
            data: JSON.stringify(datasheet.data),
            meta: JSON.stringify(datasheet.meta),
            tags: {
                connect: tagIds,
            },
            import: {
                connect: { id: this.importId },
            },
            rowKey,
            columnKey,
            tagKey,
            tagInfo: JSON.stringify(tagInfo),
        };
    }
    async deleteExistingDatasheets(createSheetData) {
        const ids = await this.prisma.sheet.deleteMany({
            where: {
                tagKey: createSheetData.tagKey,
            },
        });
        this.summary.deleted.push(ids.count);
    }
    async createSheet(datasheet, createSheetData) {
        const newSheet = await this.prisma.sheet.create({
            data: createSheetData,
        });
        if (newSheet.id) {
            datasheet.id = newSheet.id;
            this.summary.ids.push(datasheet.id);
        }
        else {
            console.error(datasheet);
            throw "Could not store datasheet.";
        }
        return newSheet;
    }
    async dedupTagKeys(sheetData) {
        const tagKey = sheetData.tagKey;
        if (!this.sheetKeys[tagKey]) {
            this.sheetKeys[tagKey] = 1;
        }
        else {
            this.sheetKeys[tagKey] += 1;
            const addCounterTags = [
                {
                    category: "iSheetKey",
                    value: "Sheet " + String(this.sheetKeys[tagKey]),
                },
            ];
            await this.addCategoryIdToTags(addCounterTags);
            await this.addTagIdsToTags(addCounterTags);
            const connectTags = [
                ...Object.values(sheetData.tags.connect),
                { id: addCounterTags[0].id },
            ];
            sheetData.tags.connect = connectTags;
            sheetData.tagKey = this.getTagKey(sheetData.tags.connect);
            console.log("Duplicate sheet key: ");
            console.log(sheetData);
        }
    }
    async addCategoryIdToTags(tags) {
        for (let i in tags) {
            const tag = tags[i];
            const cat = this.categories.find((cat) => cat.name === tag.category);
            if (cat && cat.id) {
                tag.categoryId = cat.id;
            }
            else {
                const newCat = await this.createCategory(tag.category);
                tag.categoryId = newCat.id;
            }
        }
    }
    async addTagIdsToTags(tags) {
        for (let i in tags) {
            const tag = tags[i];
            if (tag.value.length > this.tagValueMaxLength) {
                console.error("Tag name shortened: " + tag.value);
                tag.value = tag.value.slice(256);
            }
            const existingTag = this.options.tagsCache.get(tag.value, tag.categoryId);
            // const existingTag = this.tags.find(
            //   (existingTag: Tag) =>
            //     existingTag.name === tag.value &&
            //     existingTag.categoryId === tag.categoryId
            // );
            if (!existingTag) {
                if (!tag.value) {
                    console.log(tag);
                    throw "Can't add tag!";
                }
                let newTag = await this.createTag(tag.value, tag.categoryId);
                tag.id = newTag.id;
            }
            else {
                tag.id = existingTag.id;
            }
        }
    }
    getSheetKeys(datasheet) {
        const keys = {
            rowKey: this.createHashFromArray(datasheet.rows),
            columnKey: this.createHashFromArray(datasheet.columns),
            tagKey: this.getTagKey(datasheet.tags),
            sheetKey: null,
        };
        keys.sheetKey = this.createHashFromObject(keys);
        return keys;
    }
    getTagKey(tags) {
        return this.createHashFromArray(tags.map((tag) => tag.id));
    }
    createHashFromObject(obj) {
        const str = JSON.stringify(obj);
        return this.createHash(str);
    }
    createHashFromArray(arr) {
        const str = arr.join("|");
        return this.createHash(str);
    }
    createHash(str) {
        const sha256Hasher = crypto_1.default.createHmac("sha256", "123");
        const hash = sha256Hasher.update(str).digest("hex");
        return hash;
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map